import i3ipc

WORKSPACE_STACK = []
WORKSPACE_STACK_SIZE = 0


def is_debug():
    return False


def set_workspace_stack_size(size):
    global WORKSPACE_STACK_SIZE
    WORKSPACE_STACK_SIZE = size


# Define a callback to be called when you switch workspaces.
def on_workspace_focus(i3, e):
    if len(WORKSPACE_STACK) < WORKSPACE_STACK_SIZE:
        pass
    elif len(WORKSPACE_STACK) >= WORKSPACE_STACK_SIZE:
        WORKSPACE_STACK.append(e.old)
    set_workspace_stack_size(len(WORKSPACE_STACK))
    if is_debug():
        print([wspace.name for wspace in WORKSPACE_STACK])


def on_keys(i3, e):
    if e.binding.symbol == 'z' and 'Mod4' in e.binding.mods and WORKSPACE_STACK:
        if is_debug():
            print([wspace.name for wspace in WORKSPACE_STACK])
        if len(WORKSPACE_STACK) > 0:
            if is_debug():
                print('Rollback to workspace %s' % WORKSPACE_STACK[-1].name)
            i3.command('workspace %s' % WORKSPACE_STACK.pop().name)


def start():
    # Create the Connection object that can be used to send commands and subscribe
    # to events.
    i3 = i3ipc.Connection()
    # Subscribe to events
    i3.on('workspace::focus', on_workspace_focus)
    i3.on('binding', on_keys)

    # Start the main loop and wait for events to come in.
    i3.main()


if __name__ == '__main__':
    start()



import unittest
from contextlib import contextmanager

import rollback
from unittest.mock import MagicMock


@contextmanager
def rollback_event():
    mocked_event = MagicMock()
    mocked_event.binding.symbol = 'z'
    mocked_event.binding.mods = ['Mod4']
    mocked_i3 = MagicMock()
    yield mocked_i3, mocked_event
    rollback.on_keys(mocked_i3, mocked_event)  # every time a rollback is invoked, workspace_focus is triggered
    rollback.on_workspace_focus(mocked_i3, mocked_event)


@contextmanager
def workspace_focus_event():
    mocked_event = MagicMock()
    mocked_event.binding.symbol = 'z'
    mocked_event.binding.mods = ['Mod4']
    mocked_i3 = MagicMock()
    yield mocked_i3, mocked_event
    rollback.on_workspace_focus(mocked_i3, mocked_event)


class TestRollback(unittest.TestCase):
    def test_should_call_workspace_change_on_right_key_binding(self):
        with rollback_event() as (i3, event):
            mocked_worspace = MagicMock()
            mocked_worspace.name = 'random name'
            rollback.set_workspace_stack_size(0)
            rollback.WORKSPACE_STACK = [mocked_worspace]

        i3.command.assert_called_with('workspace random name')

    def test_should_on_workspace_change_add_old_workspace_to_stack(self):
        with workspace_focus_event() as (i3, event):
            mocked_worspace = MagicMock()
            mocked_worspace.name = 'random name'
            rollback.WORKSPACE_STACK = []
            rollback.set_workspace_stack_size(0)
            event = MagicMock()
            event.binding.symbol = 'z'
            event.binding.mods = ['Mod4']
            event.old = mocked_worspace

        self.assertEqual(len(rollback.WORKSPACE_STACK), 1)

    def test_should_rollback_one_workspace(self):
        with rollback_event() as (i3, event):
            mocked_worspace1 = MagicMock()
            mocked_worspace1.name = 'random name 1'
            mocked_worspace2 = MagicMock()
            mocked_worspace2.name = 'random name 2'
            mocked_worspace3 = MagicMock()
            mocked_worspace3.name = 'random name 3'
            mocked_worspace4 = MagicMock()
            mocked_worspace4.name = 'random name 4'
            rollback.WORKSPACE_STACK = [mocked_worspace1, mocked_worspace2, mocked_worspace3, mocked_worspace4]
            rollback.set_workspace_stack_size(4)

        self.assertEqual(len(rollback.WORKSPACE_STACK), 3)



from setuptools import setup

setup(name='i3-workspace-rollback',
      version='1.1',
      description='i3 plugin for rollback to last focused workspace',
      url='https://github.com/alexrochas/i3-workspace-rollback',
      author='Alex Rocha',
      author_email='alex.rochas@yahoo.com.br',
      license='MIT',
      packages=['rollback'],
      install_requires=['enum-compat==0.0.2',
                        'i3ipc==1.3.0',
                        'mock==2.0.0',
                        'nose==1.3.7',
                        'pbr==1.10.0',
                        'six==1.10.0'],
      entry_points={
          'console_scripts': ['i3-workspace-rollback=rollback.rollback:start'],
      },
      include_package_data=True,
      zip_safe=False)
